close all;
clear;
clc;
addpath('/home/michal/opt/openEMS/share/openEMS/matlab');
addpath('/home/michal/opt/openEMS/share/CSXCAD/matlab');
addpath('/home/michal/opt/openEMS/share/hyp2mat/matlab');
addpath('/home/michal/opt/openEMS/share/CTB/matlab');

###############################
## PREPARE SIMULATION MODEL
###############################

#constants
physical_constants;
impedance = 50;
fmin = 1e6;
fmax = 24e9;

#dimensions
res_coarse = 3e8/fmax/90;
res_fine = res_coarse / 3;
mesh_start_x = -25e-3;
mesh_stop_x = -mesh_start_x;
mesh_start_y = -50e-3;
mesh_stop_y = 5e-3;
mesh_start_z = -6e-3;
mesh_stop_z = -mesh_start_z;

line_w = 1.8e-3;
slot_w = 0.15e-3;
slot_open_w = 6e-3;
sub_h = 1e-3;

er = 4.5;
tan_loss = 0.02;
kap = tan_loss*2*pi*(5.5e9)*EPS0*er;

#setup FDTD
FDTD = InitFDTD('EndCriteria', 1e-5);
FDTD = SetGaussExcite(FDTD, (fmax+fmin)/2, (fmax-fmin)/2);
BC = {'PML_8' 'PML_8' 'PML_8' 'PML_8' 'PML_8' 'PML_8'};
FDTD = SetBoundaryCond(FDTD, BC);
#setup CSX
CSX = InitCSX();

#add materials
CSX = AddMetal(CSX,'Metal');
CSX = AddMetal(CSX,'Metal1');
CSX = AddMaterial(CSX, 'FR-4');
CSX = SetMaterialProperty(CSX, 'FR-4', 'Epsilon', er, 'Kappa', kap);

#geometry
CSX = ImportSTL(CSX, 'Metal1',10, '/home/michal/ELEKTRONIKA/asdf_sa/SIGN_CONV/design/1st_mixer/slot_balun/3d/sim_sparExtract.stl','Transform',{'Scale',1/1000});

start = [mesh_start_x, mesh_start_y, 0];
stop = [mesh_stop_x, mesh_stop_y, sub_h];
CSX = AddBox(CSX, 'FR-4', 5, start, stop); #substrate

start = [-0.6e-3, -29e-3, 0];
stop = [0.6e-3, -29.6e-3, 0];
CSX = AddBox(CSX, 'Metal', 5, start, stop); # to connect ports
start = [-0.2e-3, -28.2e-3, 0];
stop = [-0.5e-3, -28.5e-3, 0];
CSX = AddBox(CSX, 'Metal', 5, start, stop); # to connect ports
start = [0.2e-3, -28.2e-3, 0];
stop = [0.5e-3, -28.5e-3, 0];
CSX = AddBox(CSX, 'Metal', 5, start, stop); # to connect ports

#add ports
start = [-9.5e-3-1.8e-3/2, -0.2e-3, 0];
stop = [-9.5e-3+1.8e-3/2, -0.2e-3, sub_h];
[CSX,port{1}] = AddLumpedPort(CSX, 999, 1, impedance, start, stop, [0 0 1], true);
start = [-0.2e-3, -29e-3, 0];
stop = [-0.5e-3, -28.5e-3, 0];
[CSX,port{2}] = AddLumpedPort(CSX, 999, 2, impedance, start, stop, [0 1 0]);
start = [0.2e-3, -29e-3, 0];
stop = [0.5e-3, -28.5e-3, 0];
[CSX,port{3}] = AddLumpedPort(CSX, 999, 3, impedance, start, stop, [0 1 0]);

#CSX = AddDump(CSX,'Et');
start = [mesh_start_x, mesh_start_y, mesh_start_z];
stop = [mesh_stop_x, mesh_stop_y, mesh_stop_z];
#CSX = AddBox(CSX,'Et',900, start, stop);

#setup mesh
mesh_x1 = SmoothMeshLines([-2*slot_w, -slot_w/2-1/3*res_fine, -slot_w/2+2/3*res_fine, slot_w/2-2/3*res_fine, slot_w/2+1/3*res_fine, 2*slot_w], res_fine, 1.5);
mesh_x2 = SmoothMeshLines([-slot_open_w/2-1/3*res_coarse, -slot_open_w/2+2/3*res_coarse, mesh_x1, slot_open_w/2-2/3*res_coarse, slot_open_w/2+1/3*res_coarse], res_coarse, 1.5);
mesh.x = SmoothMeshLines([mesh_start_x, mesh_x2, mesh_stop_x], res_coarse, 1.5);
mesh_y1 = SmoothMeshLines([-29.5e-3-1/3*res_fine, -29.5e-3+2/3*res_fine, -28.2e-3-2/3*res_fine, -28.2e-3+1/3*res_fine], res_fine, 1.5);
mesh_y2 = SmoothMeshLines([-18e-3-line_w/2-2/3*res_fine, -18e-3-line_w/2+1/3*res_fine, -18e-3+line_w/2-1/3*res_fine, -18e-3+line_w/2+2/3*res_fine], res_fine, 1.5);
mesh.y = SmoothMeshLines([mesh_start_y, -0.2e-3, mesh_y1, mesh_y2, mesh_stop_y], res_coarse, 1.5);
mesh.z = SmoothMeshLines([-sub_h, 0, sub_h, 2*sub_h], res_fine, 1.5);
mesh.z = SmoothMeshLines([mesh_start_z, mesh.z, mesh_stop_z], res_coarse, 1.5);
CSX = DefineRectGrid(CSX, 1, mesh);


###############################
## PREPARE AND SIMULATE
###############################

#set files, paths, write data, show geometry, run the simulator
confirm_recursive_rmdir(0); #don't want asking when remomoving directory
sim_path = 'sign_conv_proto1_e9_sparExtract_s11_s21_s31';
sim_file = 'sim_file.xml';
#rmdir(sim_path, 's');
mkdir(sim_path);
#save structures with data about simulation to file
WriteOpenEMS([sim_path '/' sim_file], FDTD, CSX);
#run a geometry viewer
CSXGeomPlot([sim_path '/' sim_file]);
#simulate
RunOpenEMS(sim_path, sim_file);


###############################
## PROCESS RESULTS
###############################

#S11 plot
f = linspace(fmin, fmax, 10000);
port = calcPort(port, sim_path, f, 'RefImpedance', impedance);
s11 = port{1}.uf.ref ./ port{1}.uf.inc;
s21 = port{2}.uf.ref ./ port{1}.uf.inc;
s31 = port{3}.uf.ref ./ port{1}.uf.inc;
figure(1);
hold on;
plot(f/1e9, 20*log10(abs(s11)), 'r', 'LineWidth', 2);
plot(f/1e9, 20*log10(abs(s21)), 'b', 'LineWidth', 2);
plot(f/1e9, 20*log10(abs(s31)), 'g', 'LineWidth', 2);
hold off;








##############
##########     2
##############



clear CSX;
clear FDTD;

#setup FDTD
FDTD = InitFDTD('EndCriteria', 1e-4);
FDTD = SetGaussExcite(FDTD, (fmax+fmin)/2, (fmax-fmin)/2);
BC = {'PML_8' 'PML_8' 'PML_8' 'PML_8' 'PML_8' 'PML_8'};
FDTD = SetBoundaryCond(FDTD, BC);
#setup CSX
CSX = InitCSX();

#add materials
CSX = AddMetal(CSX,'Metal');
CSX = AddMetal(CSX,'Metal1');
CSX = AddMaterial(CSX, 'FR-4');
CSX = SetMaterialProperty(CSX, 'FR-4', 'Epsilon', er, 'Kappa', kap);

#geometry
CSX = ImportSTL(CSX, 'Metal1',10, '/home/michal/ELEKTRONIKA/asdf_sa/SIGN_CONV/design/1st_mixer/slot_balun/3d/sim_sparExtract.stl','Transform',{'Scale',1/1000});

start = [mesh_start_x, mesh_start_y, 0];
stop = [mesh_stop_x, mesh_stop_y, sub_h];
CSX = AddBox(CSX, 'FR-4', 5, start, stop); #substrate

start = [-0.6e-3, -29e-3, 0];
stop = [0.6e-3, -29.6e-3, 0];
CSX = AddBox(CSX, 'Metal', 5, start, stop); # to connect ports
start = [-0.2e-3, -28.2e-3, 0];
stop = [-0.5e-3, -28.5e-3, 0];
CSX = AddBox(CSX, 'Metal', 5, start, stop); # to connect ports
start = [0.2e-3, -28.2e-3, 0];
stop = [0.5e-3, -28.5e-3, 0];
CSX = AddBox(CSX, 'Metal', 5, start, stop); # to connect ports

#add ports
start = [-9.5e-3-1.8e-3/2, -0.2e-3, 0];
stop = [-9.5e-3+1.8e-3/2, -0.2e-3, sub_h];
[CSX,port{1}] = AddLumpedPort(CSX, 999, 1, impedance, start, stop, [0 0 1]);
start = [-0.2e-3, -29e-3, 0];
stop = [-0.5e-3, -28.5e-3, 0];
[CSX,port{2}] = AddLumpedPort(CSX, 999, 2, impedance, start, stop, [0 1 0], true);
start = [0.2e-3, -29e-3, 0];
stop = [0.5e-3, -28.5e-3, 0];
[CSX,port{3}] = AddLumpedPort(CSX, 999, 3, impedance, start, stop, [0 1 0]);

#CSX = AddDump(CSX,'Et');
start = [mesh_start_x, mesh_start_y, mesh_start_z];
stop = [mesh_stop_x, mesh_stop_y, mesh_stop_z];
#CSX = AddBox(CSX,'Et',900, start, stop);

#setup mesh
mesh_x1 = SmoothMeshLines([-2*slot_w, -slot_w/2-1/3*res_fine, -slot_w/2+2/3*res_fine, slot_w/2-2/3*res_fine, slot_w/2+1/3*res_fine, 2*slot_w], res_fine, 1.5);
mesh_x2 = SmoothMeshLines([-slot_open_w/2-1/3*res_coarse, -slot_open_w/2+2/3*res_coarse, mesh_x1, slot_open_w/2-2/3*res_coarse, slot_open_w/2+1/3*res_coarse], res_coarse, 1.5);
mesh.x = SmoothMeshLines([mesh_start_x, mesh_x2, mesh_stop_x], res_coarse, 1.5);
mesh_y1 = SmoothMeshLines([-29.5e-3-1/3*res_fine, -29.5e-3+2/3*res_fine, -28.2e-3-2/3*res_fine, -28.2e-3+1/3*res_fine], res_fine, 1.5);
mesh_y2 = SmoothMeshLines([-18e-3-line_w/2-2/3*res_fine, -18e-3-line_w/2+1/3*res_fine, -18e-3+line_w/2-1/3*res_fine, -18e-3+line_w/2+2/3*res_fine], res_fine, 1.5);
mesh.y = SmoothMeshLines([mesh_start_y, -0.2e-3, mesh_y1, mesh_y2, mesh_stop_y], res_coarse, 1.5);
mesh.z = SmoothMeshLines([-sub_h, 0, sub_h, 2*sub_h], res_fine, 1.5);
mesh.z = SmoothMeshLines([mesh_start_z, mesh.z, mesh_stop_z], res_coarse, 1.5);
CSX = DefineRectGrid(CSX, 1, mesh);


###############################
## PREPARE AND SIMULATE
###############################

#set files, paths, write data, show geometry, run the simulator
confirm_recursive_rmdir(0); #don't want asking when remomoving directory
sim_path = 'sign_conv_proto1_e9_sparExtract_s12_s22_s32';
sim_file = 'sim_file.xml';
#rmdir(sim_path, 's');
mkdir(sim_path);
#save structures with data about simulation to file
WriteOpenEMS([sim_path '/' sim_file], FDTD, CSX);
#run a geometry viewer
#CSXGeomPlot([sim_path '/' sim_file]);
#simulate
RunOpenEMS(sim_path, sim_file);


###############################
## PROCESS RESULTS
###############################

#S11 plot
f = linspace(fmin, fmax, 10000);
port = calcPort(port, sim_path, f, 'RefImpedance', impedance);
s12 = port{1}.uf.ref ./ port{2}.uf.inc;
s22 = port{2}.uf.ref ./ port{2}.uf.inc;
s32 = port{3}.uf.ref ./ port{2}.uf.inc;
figure(2);
hold on;
plot(f/1e9, 20*log10(abs(s12)), 'r', 'LineWidth', 2);
plot(f/1e9, 20*log10(abs(s22)), 'b', 'LineWidth', 2);
plot(f/1e9, 20*log10(abs(s32)), 'g', 'LineWidth', 2);
hold off;











##############
##########     3
##############



clear CSX;
clear FDTD;

#setup FDTD
FDTD = InitFDTD('EndCriteria', 1e-4);
FDTD = SetGaussExcite(FDTD, (fmax+fmin)/2, (fmax-fmin)/2);
BC = {'PML_8' 'PML_8' 'PML_8' 'PML_8' 'PML_8' 'PML_8'};
FDTD = SetBoundaryCond(FDTD, BC);
#setup CSX
CSX = InitCSX();

#add materials
CSX = AddMetal(CSX,'Metal');
CSX = AddMetal(CSX,'Metal1');
CSX = AddMaterial(CSX, 'FR-4');
CSX = SetMaterialProperty(CSX, 'FR-4', 'Epsilon', er, 'Kappa', kap);

#geometry
CSX = ImportSTL(CSX, 'Metal1',10, '/home/michal/ELEKTRONIKA/asdf_sa/SIGN_CONV/design/1st_mixer/slot_balun/3d/sim_sparExtract.stl','Transform',{'Scale',1/1000});

start = [mesh_start_x, mesh_start_y, 0];
stop = [mesh_stop_x, mesh_stop_y, sub_h];
CSX = AddBox(CSX, 'FR-4', 5, start, stop); #substrate

start = [-0.6e-3, -29e-3, 0];
stop = [0.6e-3, -29.6e-3, 0];
CSX = AddBox(CSX, 'Metal', 5, start, stop); # to connect ports
start = [-0.2e-3, -28.2e-3, 0];
stop = [-0.5e-3, -28.5e-3, 0];
CSX = AddBox(CSX, 'Metal', 5, start, stop); # to connect ports
start = [0.2e-3, -28.2e-3, 0];
stop = [0.5e-3, -28.5e-3, 0];
CSX = AddBox(CSX, 'Metal', 5, start, stop); # to connect ports

#add ports
start = [-9.5e-3-1.8e-3/2, -0.2e-3, 0];
stop = [-9.5e-3+1.8e-3/2, -0.2e-3, sub_h];
[CSX,port{1}] = AddLumpedPort(CSX, 999, 1, impedance, start, stop, [0 0 1]);
start = [-0.2e-3, -29e-3, 0];
stop = [-0.5e-3, -28.5e-3, 0];
[CSX,port{2}] = AddLumpedPort(CSX, 999, 2, impedance, start, stop, [0 1 0]);
start = [0.2e-3, -29e-3, 0];
stop = [0.5e-3, -28.5e-3, 0];
[CSX,port{3}] = AddLumpedPort(CSX, 999, 3, impedance, start, stop, [0 1 0], true);

#CSX = AddDump(CSX,'Et');
start = [mesh_start_x, mesh_start_y, mesh_start_z];
stop = [mesh_stop_x, mesh_stop_y, mesh_stop_z];
#CSX = AddBox(CSX,'Et',900, start, stop);

#setup mesh
mesh_x1 = SmoothMeshLines([-2*slot_w, -slot_w/2-1/3*res_fine, -slot_w/2+2/3*res_fine, slot_w/2-2/3*res_fine, slot_w/2+1/3*res_fine, 2*slot_w], res_fine, 1.5);
mesh_x2 = SmoothMeshLines([-slot_open_w/2-1/3*res_coarse, -slot_open_w/2+2/3*res_coarse, mesh_x1, slot_open_w/2-2/3*res_coarse, slot_open_w/2+1/3*res_coarse], res_coarse, 1.5);
mesh.x = SmoothMeshLines([mesh_start_x, mesh_x2, mesh_stop_x], res_coarse, 1.5);
mesh_y1 = SmoothMeshLines([-29.5e-3-1/3*res_fine, -29.5e-3+2/3*res_fine, -28.2e-3-2/3*res_fine, -28.2e-3+1/3*res_fine], res_fine, 1.5);
mesh_y2 = SmoothMeshLines([-18e-3-line_w/2-2/3*res_fine, -18e-3-line_w/2+1/3*res_fine, -18e-3+line_w/2-1/3*res_fine, -18e-3+line_w/2+2/3*res_fine], res_fine, 1.5);
mesh.y = SmoothMeshLines([mesh_start_y, -0.2e-3, mesh_y1, mesh_y2, mesh_stop_y], res_coarse, 1.5);
mesh.z = SmoothMeshLines([-sub_h, 0, sub_h, 2*sub_h], res_fine, 1.5);
mesh.z = SmoothMeshLines([mesh_start_z, mesh.z, mesh_stop_z], res_coarse, 1.5);
CSX = DefineRectGrid(CSX, 1, mesh);


###############################
## PREPARE AND SIMULATE
###############################

#set files, paths, write data, show geometry, run the simulator
confirm_recursive_rmdir(0); #don't want asking when remomoving directory
sim_path = 'sign_conv_proto1_e9_sparExtract_s13_s23_s33';
sim_file = 'sim_file.xml';
#rmdir(sim_path, 's');
mkdir(sim_path);
#save structures with data about simulation to file
WriteOpenEMS([sim_path '/' sim_file], FDTD, CSX);
#run a geometry viewer
#CSXGeomPlot([sim_path '/' sim_file]);
#simulate
RunOpenEMS(sim_path, sim_file);


###############################
## PROCESS RESULTS
###############################

#S11 plot
f = linspace(fmin, fmax, 10000);
port = calcPort(port, sim_path, f, 'RefImpedance', impedance);
s13 = port{1}.uf.ref ./ port{3}.uf.inc;
s23 = port{2}.uf.ref ./ port{3}.uf.inc;
s33 = port{3}.uf.ref ./ port{3}.uf.inc;
figure(3);
hold on;
plot(f/1e9, 20*log10(abs(s13)), 'r', 'LineWidth', 2);
plot(f/1e9, 20*log10(abs(s23)), 'b', 'LineWidth', 2);
plot(f/1e9, 20*log10(abs(s33)), 'g', 'LineWidth', 2);
hold off;




##########################
##########################
##########################
##########################


spara(1,1,:) = s11;
spara(2,1,:) = s21;
spara(3,1,:) = s31;
spara(1,2,:) = s12;
spara(2,2,:) = s22;
spara(3,2,:) = s32;
spara(1,3,:) = s13;
spara(2,3,:) = s23;
spara(3,3,:) = s33;
write_touchstone('s',f,spara,"balun.s3p");
