close all;
clear;
clc;
addpath('C:/openEMS/matlab');
file_path = fileparts(mfilename('fullpath'));
cd(file_path);
###############################
## PREPARE SIMULATION MODEL
###############################

#constants
physical_constants;
impedance = 50;
fmin = 2e9;
fmax = 10e9;

#dimensions
res_coarse = 3e8/fmax/50;
res_fine = res_coarse / 4;
mesh_start_x = -18e-3;
mesh_stop_x = -mesh_start_x;
mesh_start_y = -18e-3;
mesh_stop_y = -mesh_start_y;
mesh_start_z = -6e-3;
mesh_stop_z = -mesh_start_z;

line_w = 1.8e-3;
slot_w = 0.15e-3;
sub_h = 1e-3;

#setup FDTD
FDTD = InitFDTD('EndCriteria', 1e-4);
FDTD = SetGaussExcite(FDTD, (fmax+fmin)/2, (fmax-fmin)/2);
BC = {'PML_8' 'PML_8' 'PML_8' 'PML_8' 'PML_8' 'PML_8'};
FDTD = SetBoundaryCond(FDTD, BC);
#setup CSX
CSX = InitCSX();

#add materials
CSX = AddMetal(CSX,'Metal');
er = 4.5;
tan_loss = 0.02;
kap = tan_loss*2*pi*((fmin*fmax)^0.5)*EPS0*er;
CSX = AddMaterial(CSX, 'FR-4');
CSX = SetMaterialProperty(CSX, 'FR-4', 'Epsilon', er, 'Kappa', kap);

#geometry
start = [mesh_start_x, mesh_start_y, 0];
stop = [mesh_stop_x, mesh_stop_y, sub_h];
CSX = AddBox(CSX, 'FR-4', 5, start, stop); #substrate
start = [-10e-3, -line_w/2, sub_h];
stop = [7.4e-3, line_w/2, sub_h];
CSX = AddBox(CSX, 'Metal', 5, start, stop); #line
start = [mesh_start_x, mesh_start_y, 0];
stop = [-slot_w/2, 7.4e-3, 0];
CSX = AddBox(CSX, 'Metal', 5, start, stop); #slot part 1
start = [mesh_stop_x, mesh_start_y, 0];
stop = [slot_w/2, 7.4e-3, 0];
CSX = AddBox(CSX, 'Metal', 5, start, stop); #slot part 2
start = [mesh_start_x, 7.4e-3, 0];
stop = [mesh_stop_x, mesh_stop_y, 0];
CSX = AddBox(CSX, 'Metal', 5, start, stop); #slot part 3
#
start = [mesh_start_x, -11e-3, 0];
stop = [mesh_stop_x, mesh_start_y, 0];
#CSX = AddBox(CSX, 'Metal', 5, start, stop);
start = [mesh_start_x, mesh_start_y, 0];
stop = [-2e-3, mesh_stop_y, 0];
#CSX = AddBox(CSX, 'Metal', 5, start, stop);
start = [mesh_stop_x, mesh_start_y, 0];
stop = [2e-3, mesh_stop_y, 0];
#CSX = AddBox(CSX, 'Metal', 5, start, stop);
#start = [-0.5e-3, mesh_start_y, 0];
#stop = [0.5e-3, mesh_stop_y, -sub_h];
#CSX = AddBox(CSX, 'FR-4', 4, start, stop); #substrate

#add ports
start = [-10e-3, -line_w/2, 0];
stop = [-10e-3, line_w/2, sub_h];
[CSX,port{1}] = AddLumpedPort(CSX, 999, 1, impedance, start, stop, [0 0 1], true);
start = [-slot_w/2, -10e-3-0.2e-3, 0];
stop = [slot_w/2, -10e-3+0.2e-3, 0];
#[CSX,port{2}] = AddLumpedPort(CSX, 999, 2, impedance, start, stop, [1 0 0]);

#CSX = AddDump(CSX,'Et');
#start = [mesh_start_x, mesh_start_y, mesh_start_z];
#stop = [mesh_stop_x, mesh_stop_y, mesh_stop_z];
#CSX = AddBox(CSX,'Et',900, start, stop);

#setup mesh
mesh.x = SmoothMeshLines([-2*slot_w, -slot_w, slot_w, 2*slot_w], res_fine/3, 1.5);
mesh.x = SmoothMeshLines([mesh_start_x, -10e-3, mesh.x, mesh_stop_x], res_coarse, 1.5);
mesh_y1 = SmoothMeshLines([-10e-3-0.2e-3, -10e-3+0.2e-3], res_fine, 1.5);
mesh_y2 = SmoothMeshLines([-2*line_w, 2*line_w], res_fine, 1.5);
mesh.y = SmoothMeshLines([mesh_start_y, mesh_y1, mesh_y2, mesh_stop_y], res_coarse, 1.5);
mesh.z = SmoothMeshLines([-sub_h, 0, sub_h, 2*sub_h], res_fine, 1.5);
mesh.z = SmoothMeshLines([mesh_start_z, mesh.z, mesh_stop_z], res_coarse, 1.5);
CSX = DefineRectGrid(CSX, 1, mesh);


###############################
## PREPARE AND SIMULATE
###############################

#set files, paths, write data, show geometry, run the simulator
confirm_recursive_rmdir(0); #don't want asking when remomoving directory
sim_path = 'sim_path';
sim_file = 'sim_file.xml';
#rmdir(sim_path, 's');
mkdir(sim_path);
#save structures with data about simulation to file
WriteOpenEMS([sim_path '/' sim_file], FDTD, CSX);
#run a geometry viewer
CSXGeomPlot([sim_path '/' sim_file]);
#simulate
RunOpenEMS(sim_path, sim_file);


###############################
## PROCESS RESULTS
###############################

#S11 plot
f = linspace(fmin, fmax, 2000);
port = calcPort(port, sim_path, f, 'RefImpedance', impedance);
s11 = port{1}.uf.ref ./ port{1}.uf.inc;
#s21 = port{2}.uf.ref ./ port{1}.uf.inc;
figure(1);
hold on;
plot(f/1e9, 20*log10(abs(s11)), 'r', 'LineWidth', 2);
#plot(f/1e9, 20*log10(abs(s21)), 'b', 'LineWidth', 2);
hold off;

plotRefl(port{1}, 'fmarkers', [4e9, 7e9])
