close all
clear
clc

addpath('/opt/openEMS/share/openEMS/matlab');
addpath('/opt/openEMS/share/CSXCAD/matlab');
addpath('/opt/openEMS/share/hyp2mat/matlab');
addpath('/opt/openEMS/share/CTB/matlab');
physical_constants; #import physical_constants

###############################
## PREPARE SIMULATION MODEL
###############################

#variables

fmin = 1e9;
fmax = 16e9;
impedance = 50;
eps_r = 4.5; #FR-4 permability
tan_loss = 0.02; #FR-4 losses (as tangens)
kap = tan_loss*2*pi*((fmin*fmax)^0.5)*EPS0*eps_r; #FR-4 losses (as kappa)
res = c0 / (fmax*sqrt(eps_r)) / 60;
res_fine = res / 3;

subst_h = 1e-3;
subs_l = 40e-3;
subs_w = 8e-3;
line_w = 0.2e-3;
feed_w = 1.8e-3;
l1 = 3.5e-3*1.1;
l2 = 5.8e-3*1.1;
l3 = 6e-3*1.1;
s1 = 2.0e-3;
s2 = 2.1e-3;
stub_cw = 1e-3;
boxsize_x = subs_l*1.2;
boxsize_y = subs_w*1.1;
boxsize_z = subst_h*5;

#initiate and setup simulation structures and parameters
FDTD = InitFDTD('EndCriteria', 1e-5);
FDTD = SetGaussExcite(FDTD, fmin, fmax);
BC = {'PML_8' 'PML_8' 'PML_8' 'PML_8' 'PEC' 'PML_8'};
FDTD = SetBoundaryCond(FDTD, BC);
CSX = InitCSX();

#definitions of used materials
CSX = AddMaterial(CSX, 'FR4');
CSX = SetMaterialProperty(CSX, 'FR4', 'Epsilon', eps_r, 'Kappa', kap);
#CSX = AddConductingSheet(CSX,'Copper', 56e6, 16e-6);
CSX = AddMetal(CSX,'Metal', 56e6, 16e-6);


#geometry draw:
CSX = AddMRStub(CSX, 'Metal', 100, stub_cw, s1, 120, 1, -90, [0 0 1], [-l2-l3/2-3*stub_cw/2, -line_w/2, subst_h]);
CSX = AddMRStub(CSX, 'Metal', 100, stub_cw, s1, 120, 1, 90, [0 0 1], [-l2-l3/2-3*stub_cw/2, line_w/2, subst_h]);
CSX = AddMRStub(CSX, 'Metal', 100, stub_cw, s2, 120, 1, -90, [0 0 1], [-l3/2-stub_cw/2, -line_w/2, subst_h]);
CSX = AddMRStub(CSX, 'Metal', 100, stub_cw, s2, 120, 1, 90, [0 0 1], [-l3/2-stub_cw/2, line_w/2, subst_h]);
CSX = AddMRStub(CSX, 'Metal', 100, stub_cw, s2, 120, 1, -90, [0 0 1], [l3/2+stub_cw/2, -line_w/2, subst_h]);
CSX = AddMRStub(CSX, 'Metal', 100, stub_cw, s2, 120, 1, 90, [0 0 1], [l3/2+stub_cw/2, line_w/2, subst_h]);
CSX = AddMRStub(CSX, 'Metal', 100, stub_cw, s1, 120, 1, -90, [0 0 1], [l2+l3/2+3*stub_cw/2, -line_w/2, subst_h]);
CSX = AddMRStub(CSX, 'Metal', 100, stub_cw, s1, 120, 1, 90, [0 0 1], [l2+l3/2+3*stub_cw/2, line_w/2, subst_h]);
start = [-l3/2-stub_cw-l2-stub_cw-l1, -line_w/2, subst_h];
stop = [l3/2+stub_cw+l2+stub_cw+l1, line_w/2, subst_h];
CSX = AddBox(CSX, 'Metal', 100, start, stop);
start = [-l3/2-stub_cw-l2-stub_cw-l1, -feed_w/2, subst_h];
stop = [-subs_l/2, feed_w/2, subst_h];
CSX = AddBox(CSX, 'Metal', 100, start, stop);
start = [l3/2+stub_cw+l2+stub_cw+l1, -feed_w/2, subst_h];
stop = [subs_l/2, feed_w/2, subst_h];
CSX = AddBox(CSX, 'Metal', 100, start, stop);
#substrate
start = [-subs_l/2, -subs_w/2, 0];
stop = [subs_l/2, subs_w/2, subst_h];
CSX = AddBox(CSX, 'FR4', 10, start, stop);
#ground
start = [-subs_l, -subs_w/2, 0];
stop = [0, subs_w/2, 0];
#CSX = AddBox(CSX, 'Copper', 500, start, stop);


#setup mesh
mesh.x = SmoothMeshLines([-subs_l/2, subs_l/2], res_fine, 1.5);
mesh.x = SmoothMeshLines([-boxsize_x/2, mesh.x, boxsize_x/2], res, 1.5);
mesh.y = SmoothMeshLines([-line_w/2-2/3*res_fine, -line_w/2+1/3*res_fine, line_w/2+2/3*res_fine, line_w/2-1/3*res_fine], res_fine, 1.5);
mesh.y = SmoothMeshLines([-subs_w/2, mesh.y, subs_w/2], res_fine, 1.5);
mesh.y = SmoothMeshLines([-boxsize_y/2, mesh.y, boxsize_y/2], res, 1.5);
mesh.z = SmoothMeshLines([0, subst_h, 2*subst_h], res_fine, 1.5);
mesh.z = SmoothMeshLines([mesh.z, boxsize_z], res, 1.5);
CSX = DefineRectGrid(CSX, 1, mesh);


#add ports
start = [-subs_l/2, -feed_w/2, 0];
stop = [-subs_l/2, feed_w/2, subst_h];
[CSX,port{1}] = AddLumpedPort(CSX, 999, 1, impedance, start, stop, [0 0 1], true);
start = [subs_l/2, -feed_w/2, 0];
stop = [subs_l/2, feed_w/2, subst_h];
[CSX,port{2}] = AddLumpedPort(CSX, 999, 2, impedance, start, stop, [0 0 1]);


###############################
## PREPARE AND SIMULATE
###############################

#set files, paths, write data, show geometry, run the simulator
confirm_recursive_rmdir(0); #don't want asking when remomoving directory
sim_path = 'sim_path';
sim_file = 'sim_file.xml';
rmdir(sim_path, 's');
mkdir(sim_path);
#save structures with data about simulation to file
WriteOpenEMS([sim_path '/' sim_file], FDTD, CSX);
#run a geometry viewer
CSXGeomPlot([sim_path '/' sim_file]);
#simulate
RunOpenEMS(sim_path, sim_file);


###############################
## PROCESS RESULTS
###############################

f = linspace(fmin, fmax, 2000);
port = calcPort(port, sim_path, f, 'RefImpedance', impedance);
s11 = port{1}.uf.ref ./ port{1}.uf.inc;
s21 = port{2}.uf.ref ./ port{1}.uf.inc;
figure(1);
hold on;
grid on;
plot(f/1e9, 20*log10(abs(s11)), 'k-', 'LineWidth', 2);
plot(f/1e9, 20*log10(abs(s21)), 'r-', 'LineWidth', 2);

legend('S_{11}','S_{21}');
ylabel('S-Parameter (dB)','FontSize',12);
xlabel('frequency (GHz) \rightarrow','FontSize',12);

figure(2);
polar(arg(s11), abs(s11));

spara(1,1,:) = s11;
spara(2,1,:) = s21;
spara(1,2,:) = s21;
spara(2,2,:) = s11;
write_touchstone('s',f,spara,"lpf.s2p");