close all
clear
clc

addpath('/home/michal/opt/openEMS/share/openEMS/matlab');
addpath('/home/michal/opt/openEMS/share/CSXCAD/matlab');
addpath('/home/michal/opt/openEMS/share/hyp2mat/matlab');
addpath('/home/michal/opt/openEMS/share/CTB/matlab');
physical_constants; #import physical_constants

###############################
## PREPARE SIMULATION MODEL
###############################

#variables

fmin = 0.5e9;
fmax = 10e9;
impedance = 50;
eps_r = 4.5; #FR-4 permability
tan_loss = 0.02; #FR-4 losses (as tangens)
kap = tan_loss*2*pi*((fmin*fmax)^0.5)*EPS0*eps_r; #FR-4 losses (as kappa)
res = c0 / (fmax*sqrt(eps_r)) / 40;
res_f = res / 4;

subst_h = 1e-3;
boxsize_x = 50e-3;
boxsize_y = 22e-3;
boxsize_z = subst_h*5;


l_line_w = 0.2e-3;
l1 = 3.32e-3;
l2 = 5.05e-3;
l3 = 3.35e-3;
l4 = 2.90e-3;
l5 = 3.98e-3;
l6 = 2.46e-3;

r1_line_w = 0.2e-3;
r1_line_l = 0.39e-3*0.513;
r1_cap_w = 6e-3;
r1_cap_l = 3.5e-3;

r2_line_w = 0.2e-3;
r2_line_l = 2.41e-3;
r2_cap_w = 4.5e-3;
r2_cap_l = 2.6e-3;

r3_line_w = 0.2e-3;
r3_line_l = 5.68e-3;
r3_cap_w = 3.2e-3;
r3_cap_l = 1.5e-3;

r4_line_w = 0.2e-3;
r4_line_l = 4.38e-3;
r4_cap_w = 3.8e-3;
r4_cap_l = 1.7e-3;

r5_line_w = 0.2e-3;
r5_line_l = 1.29e-3;
r5_cap_w = 5.5e-3;
r5_cap_l = 2.5e-3;

feed_w = 1.8e-3;
feed_l = 3e-3;
taper_w = (feed_w - l_line_w) / 2;
flen = l1 + r1_line_w + l2 + r2_line_w + l3 + r3_line_w + l4 + r4_line_w + l5 + r5_line_w + l6 + (feed_l + taper_w)* 2;

feed1_x = -flen/2;
l1_x = feed1_x + (feed_l+taper_w);
r1_x = l1_x + l1 + r1_line_w/2;
l2_x = r1_x + r1_line_w/2;
r2_x = l2_x + l2 + r2_line_w/2;
l3_x = r2_x + r2_line_w/2;
r3_x = l3_x + l3 + r3_line_w/2;
l4_x = r3_x + r3_line_w/2;
r4_x = l4_x + l4 + r4_line_w/2;
l5_x = r4_x + r4_line_w/2;
r5_x = l5_x + l5 + r5_line_w/2;
l6_x = r5_x + r5_line_w/2;
feed2_x = l6_x + l6 + taper_w;


#initiate and setup simulation structures and parameters
FDTD = InitFDTD( 'EndCriteria', 1e-4);
FDTD = SetGaussExcite(FDTD, fmin, fmax);
BC = {'PML_8' 'PML_8' 'PML_8' 'PML_8' 'PEC' 'PML_8'};
FDTD = SetBoundaryCond(FDTD, BC);
CSX = InitCSX();

#definitions of used materials
CSX = AddMaterial(CSX, 'FR4');
CSX = SetMaterialProperty(CSX, 'FR4', 'Epsilon', eps_r, 'Kappa', kap);
CSX = AddMetal(CSX,'Metal', 56e6, 16e-6);




#geometry - filter
start = [l1_x, -l_line_w/2, subst_h];
stop = [l1_x+l1, l_line_w/2, subst_h];
#CSX = AddBox(CSX, 'Metal', 500, start, stop);

start = [r1_x-r1_line_w/2, l_line_w/2+r1_line_l, subst_h];
stop = [r1_x+r1_line_w/2, l_line_w/2, subst_h];
#CSX = AddBox(CSX, 'Metal', 500, start, stop);
start = [r1_x-r1_cap_w/2, l_line_w/2+r1_line_l, subst_h];
stop = [r1_x+r1_cap_w/2, l_line_w/2+r1_line_l+r1_cap_l, subst_h];
#CSX = AddBox(CSX, 'Metal', 500, start, stop);
start = [r1_x-r1_line_w/2, -l_line_w/2-r1_line_l, subst_h];
stop = [r1_x+r1_line_w/2, -l_line_w/2, subst_h];
CSX = AddBox(CSX, 'Metal', 500, start, stop);
start = [r1_x-r1_cap_w/2, -l_line_w/2-r1_line_l, subst_h];
stop = [r1_x+r1_cap_w/2, -l_line_w/2-r1_line_l-r1_cap_l, subst_h];
CSX = AddBox(CSX, 'Metal', 500, start, stop);

start = [l2_x, -l_line_w/2, subst_h];
stop = [l2_x+l2, l_line_w/2, subst_h];
#CSX = AddBox(CSX, 'Metal', 500, start, stop);

start = [r2_x-r2_line_w/2, l_line_w/2+r2_line_l, subst_h];
stop = [r2_x+r2_line_w/2, l_line_w/2, subst_h];
CSX = AddBox(CSX, 'Metal', 500, start, stop);
start = [r2_x-r2_cap_w/2, l_line_w/2+r2_line_l, subst_h];
stop = [r2_x+r2_cap_w/2, l_line_w/2+r2_line_l+r2_cap_l, subst_h];
CSX = AddBox(CSX, 'Metal', 500, start, stop);

start = [l3_x, -l_line_w/2, subst_h];
stop = [l3_x+l3, l_line_w/2, subst_h];
#CSX = AddBox(CSX, 'Metal', 500, start, stop);

start = [r3_x-r3_line_w/2, -l_line_w/2-r3_line_l, subst_h];
stop = [r3_x+r3_line_w/2, -l_line_w/2, subst_h];
CSX = AddBox(CSX, 'Metal', 500, start, stop);
start = [r3_x-r3_cap_w/2, -l_line_w/2-r3_line_l, subst_h];
stop = [r3_x+r3_cap_w/2, -l_line_w/2-r3_line_l-r3_cap_l, subst_h];
CSX = AddBox(CSX, 'Metal', 500, start, stop);

start = [l4_x, -l_line_w/2, subst_h];
stop = [l4_x+l4, l_line_w/2, subst_h];
#CSX = AddBox(CSX, 'Metal', 500, start, stop);

start = [r4_x-r4_line_w/2, l_line_w/2+r4_line_l, subst_h];
stop = [r4_x+r4_line_w/2, l_line_w/2, subst_h];
CSX = AddBox(CSX, 'Metal', 500, start, stop);
start = [r4_x-r4_cap_w/2, l_line_w/2+r4_line_l, subst_h];
stop = [r4_x+r4_cap_w/2, l_line_w/2+r4_line_l+r4_cap_l, subst_h];
CSX = AddBox(CSX, 'Metal', 500, start, stop);

start = [l5_x, -l_line_w/2, subst_h];
stop = [l5_x+l5, l_line_w/2, subst_h];
#CSX = AddBox(CSX, 'Metal', 500, start, stop);

start = [r5_x-r5_line_w/2, -l_line_w/2-r5_line_l, subst_h];
stop = [r5_x+r5_line_w/2, -l_line_w/2, subst_h];
CSX = AddBox(CSX, 'Metal', 500, start, stop);
start = [r5_x-r5_cap_w/2, -l_line_w/2-r5_line_l, subst_h];
stop = [r5_x+r5_cap_w/2, -l_line_w/2-r5_line_l-r5_cap_l, subst_h];
CSX = AddBox(CSX, 'Metal', 500, start, stop);

start = [l6_x, -l_line_w/2, subst_h];
stop = [l6_x+l6, l_line_w/2, subst_h];
#CSX = AddBox(CSX, 'Metal', 500, start, stop);


start = [feed1_x, -l_line_w/2, subst_h];
stop = [feed1_x+flen, l_line_w/2, subst_h];
CSX = AddBox(CSX, 'Metal', 500, start, stop);

#geometry - feed
start = [feed1_x, -feed_w/2, subst_h];
stop = [feed1_x+feed_l, feed_w/2, subst_h];
CSX = AddBox(CSX, 'Metal', 500, start, stop);
start = [feed2_x, -feed_w/2, subst_h];
stop = [feed2_x+feed_l, feed_w/2, subst_h];
CSX = AddBox(CSX, 'Metal', 500, start, stop);

taper(1,1) = feed1_x+feed_l;
taper(2,1) = -feed_w/2;
taper(1,2) = feed1_x+feed_l;
taper(2,2) = feed_w/2;
taper(1,3) = feed1_x+feed_l+taper_w;
taper(2,3) = l_line_w/2;
taper(1,4) = feed1_x+feed_l+taper_w;
taper(2,4) = -l_line_w/2;
CSX = AddPolygon(CSX, 'Metal', 500, 'z', subst_h, taper);
taper(1,1) = feed2_x;
taper(2,1) = -feed_w/2;
taper(1,2) = feed2_x;
taper(2,2) = feed_w/2;
taper(1,3) = feed2_x-taper_w;
taper(2,3) = l_line_w/2;
taper(1,4) = feed2_x-taper_w;
taper(2,4) = -l_line_w/2;
CSX = AddPolygon(CSX, 'Metal', 500, 'z', subst_h, taper);

#substrate
start = [-boxsize_x/2, -boxsize_y/2, 0];
stop = [boxsize_x/2, boxsize_y/2, subst_h];
CSX = AddBox(CSX, 'FR4', 400, start, stop);

#setup mesh
xr1a = r1_x-r1_cap_w/2;
xr1b = r1_x+r1_cap_w/2;
xr1la = r1_x-r1_line_w/2;
xr1lb = r1_x+r1_line_w/2;
xr2a = r2_x-r2_cap_w/2;
xr2b = r2_x+r2_cap_w/2;
xr2la = r2_x-r2_line_w/2;
xr2lb = r2_x+r2_line_w/2;
xr3a = r3_x-r3_cap_w/2;
xr3b = r3_x+r3_cap_w/2;
xr3la = r3_x-r3_line_w/2;
xr3lb = r3_x+r3_line_w/2;
xr4a = r4_x-r4_cap_w/2;
xr4b = r4_x+r4_cap_w/2;
xr4la = r4_x-r4_line_w/2;
xr4lb = r4_x+r4_line_w/2;
xr5a = r5_x-r5_cap_w/2;
xr5b = r5_x+r5_cap_w/2;
xr5la = r5_x-r5_line_w/2;
xr5lb = r5_x+r5_line_w/2;
mesh.x = SmoothMeshLines([-flen/2, flen/2, ...
xr1a-2/3*res_f, xr1a+1/3*res_f, xr1b-1/3*res_f, xr1b+2/3*res_f, ...
xr2a-2/3*res_f, xr2a+1/3*res_f, xr2b-1/3*res_f, xr2b+2/3*res_f, ...
xr3a-2/3*res_f, xr3a+1/3*res_f, xr3b-1/3*res_f, xr3b+2/3*res_f, ...
xr4a-2/3*res_f, xr4a+1/3*res_f, xr4b-1/3*res_f, xr4b+2/3*res_f, ...
xr5a-2/3*res_f, xr5a+1/3*res_f, xr5b-1/3*res_f, xr5b+2/3*res_f, ...
xr1la-2/3*res_f, xr1la+1/3*res_f, xr1lb-1/3*res_f, xr1lb+2/3*res_f, ...
xr2la-2/3*res_f, xr2la+1/3*res_f, xr2lb-1/3*res_f, xr2lb+2/3*res_f, ...
xr3la-2/3*res_f, xr3la+1/3*res_f, xr3lb-1/3*res_f, xr3lb+2/3*res_f, ...
xr4la-2/3*res_f, xr4la+1/3*res_f, xr4lb-1/3*res_f, xr4lb+2/3*res_f, ...
xr5la-2/3*res_f, xr5la+1/3*res_f, xr5lb-1/3*res_f, xr5lb+2/3*res_f, ...
], res_f, 1.5);
mesh.x = SmoothMeshLines([-boxsize_x/2, mesh.x, boxsize_x/2], res, 1.5);

lwu = l_line_w/2;
lwl = -lwu;
r1a = lwl-r1_line_l;
r1b = r1a-r1_cap_l;
r2a = lwu+r2_line_l;
r2b = r2a+r2_cap_l;
r3a = lwl-r3_line_l;
r3b = r3a-r3_cap_l;
r4a = lwu+r4_line_l;
r4b = r4a+r4_cap_l;
r5a = lwl-r5_line_l;
r5b = r5a-r5_cap_l;
yrmin = min([r1a, r1b, r2a, r2b, r3a, r3b, r4a, r4b]);
yrmax = max([r1a, r1b, r2a, r2b, r3a, r3b, r4a, r4b]);
mesh.y = SmoothMeshLines([lwl-2/3*res_f, lwl+1/3*res_f, lwu+2/3*res_f, lwu-1/3*res_f, ...
                      r1a-1/3*res_f, r1a+2/3*res_f, ...#r1b-2/3*res_f, r1b+1/3*res_f, ...
                      r2a-2/3*res_f, r2a+1/3*res_f, r2b-1/3*res_f, r2b+2/3*res_f, ...
                      r3a-1/3*res_f, r3a+2/3*res_f, r3b-2/3*res_f, r3b+1/3*res_f, ...
                      r4a-2/3*res_f, r4a+1/3*res_f, r4b-1/3*res_f, r4b+2/3*res_f, ...
                      r5a-1/3*res_f, r5a+2/3*res_f, r5b-2/3*res_f, r5b+1/3*res_f, ...
                      yrmin-0.5e-3, yrmax+0.5e-3], res_f, 1.5);
mesh.y = SmoothMeshLines([-boxsize_y/2, mesh.y, boxsize_y/2], res, 1.5);
mesh.z = SmoothMeshLines([0, subst_h, 2*subst_h], res_f, 1.5);
mesh.z = SmoothMeshLines([mesh.z, boxsize_z], res, 1.5);
CSX = DefineRectGrid(CSX, 1, mesh);


#add ports
CSX = AddMetal( CSX, 'PEC' );
portstart = [ mesh.x(1), -feed_w/2, subst_h];
portstop  = [ -flen/2,  feed_w/2, 0];
[CSX,port{1}] = AddMSLPort( CSX, 999, 1, 'PEC', portstart, portstop, 0, [0 0 -1], 'ExcitePort', 'excite', 'FeedShift', 10*res, 'MeasPlaneShift', boxsize_x/2-flen/2);
portstart = [ mesh.x(end), -feed_w/2, subst_h];
portstop  = [ flen/2,  feed_w/2, 0];
[CSX,port{2}] = AddMSLPort( CSX, 999, 2, 'PEC', portstart, portstop, 0, [0 0 -1], 'MeasPlaneShift', boxsize_x/2-flen/2);


###############################
## PREPARE AND SIMULATE
###############################

#set files, paths, write data, show geometry, run the simulator
confirm_recursive_rmdir(0); #don't want asking when remomoving directory
sim_path = 'sim_path_filter_1_a_';
sim_file = 'sim_file.xml';
rmdir(sim_path, 's');
mkdir(sim_path);
#save structures with data about simulation to file
WriteOpenEMS([sim_path '/' sim_file], FDTD, CSX);
#run a geometry viewer
CSXGeomPlot([sim_path '/' sim_file]);
#simulate
RunOpenEMS(sim_path, sim_file);


###############################
## PROCESS RESULTS
###############################

f = linspace(fmin, fmax, 2000);
port = calcPort(port, sim_path, f, 'RefImpedance', impedance);
s11 = port{1}.uf.ref./ port{1}.uf.inc;
s21 = port{2}.uf.ref./ port{1}.uf.inc;
plot(f/1e9,20*log10(abs(s11)),'k-','LineWidth',2);
hold on;
grid on;
plot(f/1e9,20*log10(abs(s21)),'r-','LineWidth',2);
legend('S_{11}','S_{21}');
ylabel('S-Parameter (dB)','FontSize',12);
xlabel('frequency (GHz) \rightarrow','FontSize',12);
ylim([-70 0]);





##################
## 2
##################
clear CSX;
clear FDTD;

#initiate and setup simulation structures and parameters
FDTD = InitFDTD( 'EndCriteria', 1e-4);
FDTD = SetGaussExcite(FDTD, fmin, fmax);
BC = {'PML_8' 'PML_8' 'PML_8' 'PML_8' 'PEC' 'PML_8'};
FDTD = SetBoundaryCond(FDTD, BC);
CSX = InitCSX();

#definitions of used materials
CSX = AddMaterial(CSX, 'FR4');
CSX = SetMaterialProperty(CSX, 'FR4', 'Epsilon', eps_r, 'Kappa', kap);
CSX = AddMetal(CSX,'Metal', 56e6, 16e-6);




#geometry - filter
start = [l1_x, -l_line_w/2, subst_h];
stop = [l1_x+l1, l_line_w/2, subst_h];
#CSX = AddBox(CSX, 'Metal', 500, start, stop);

start = [r1_x-r1_line_w/2, l_line_w/2+r1_line_l, subst_h];
stop = [r1_x+r1_line_w/2, l_line_w/2, subst_h];
#CSX = AddBox(CSX, 'Metal', 500, start, stop);
start = [r1_x-r1_cap_w/2, l_line_w/2+r1_line_l, subst_h];
stop = [r1_x+r1_cap_w/2, l_line_w/2+r1_line_l+r1_cap_l, subst_h];
#CSX = AddBox(CSX, 'Metal', 500, start, stop);
start = [r1_x-r1_line_w/2, -l_line_w/2-r1_line_l, subst_h];
stop = [r1_x+r1_line_w/2, -l_line_w/2, subst_h];
CSX = AddBox(CSX, 'Metal', 500, start, stop);
start = [r1_x-r1_cap_w/2, -l_line_w/2-r1_line_l, subst_h];
stop = [r1_x+r1_cap_w/2, -l_line_w/2-r1_line_l-r1_cap_l, subst_h];
CSX = AddBox(CSX, 'Metal', 500, start, stop);

start = [l2_x, -l_line_w/2, subst_h];
stop = [l2_x+l2, l_line_w/2, subst_h];
#CSX = AddBox(CSX, 'Metal', 500, start, stop);

start = [r2_x-r2_line_w/2, l_line_w/2+r2_line_l, subst_h];
stop = [r2_x+r2_line_w/2, l_line_w/2, subst_h];
CSX = AddBox(CSX, 'Metal', 500, start, stop);
start = [r2_x-r2_cap_w/2, l_line_w/2+r2_line_l, subst_h];
stop = [r2_x+r2_cap_w/2, l_line_w/2+r2_line_l+r2_cap_l, subst_h];
CSX = AddBox(CSX, 'Metal', 500, start, stop);

start = [l3_x, -l_line_w/2, subst_h];
stop = [l3_x+l3, l_line_w/2, subst_h];
#CSX = AddBox(CSX, 'Metal', 500, start, stop);

start = [r3_x-r3_line_w/2, -l_line_w/2-r3_line_l, subst_h];
stop = [r3_x+r3_line_w/2, -l_line_w/2, subst_h];
CSX = AddBox(CSX, 'Metal', 500, start, stop);
start = [r3_x-r3_cap_w/2, -l_line_w/2-r3_line_l, subst_h];
stop = [r3_x+r3_cap_w/2, -l_line_w/2-r3_line_l-r3_cap_l, subst_h];
CSX = AddBox(CSX, 'Metal', 500, start, stop);

start = [l4_x, -l_line_w/2, subst_h];
stop = [l4_x+l4, l_line_w/2, subst_h];
#CSX = AddBox(CSX, 'Metal', 500, start, stop);

start = [r4_x-r4_line_w/2, l_line_w/2+r4_line_l, subst_h];
stop = [r4_x+r4_line_w/2, l_line_w/2, subst_h];
CSX = AddBox(CSX, 'Metal', 500, start, stop);
start = [r4_x-r4_cap_w/2, l_line_w/2+r4_line_l, subst_h];
stop = [r4_x+r4_cap_w/2, l_line_w/2+r4_line_l+r4_cap_l, subst_h];
CSX = AddBox(CSX, 'Metal', 500, start, stop);

start = [l5_x, -l_line_w/2, subst_h];
stop = [l5_x+l5, l_line_w/2, subst_h];
#CSX = AddBox(CSX, 'Metal', 500, start, stop);

start = [r5_x-r5_line_w/2, -l_line_w/2-r5_line_l, subst_h];
stop = [r5_x+r5_line_w/2, -l_line_w/2, subst_h];
CSX = AddBox(CSX, 'Metal', 500, start, stop);
start = [r5_x-r5_cap_w/2, -l_line_w/2-r5_line_l, subst_h];
stop = [r5_x+r5_cap_w/2, -l_line_w/2-r5_line_l-r5_cap_l, subst_h];
CSX = AddBox(CSX, 'Metal', 500, start, stop);

start = [l6_x, -l_line_w/2, subst_h];
stop = [l6_x+l6, l_line_w/2, subst_h];
#CSX = AddBox(CSX, 'Metal', 500, start, stop);


start = [feed1_x, -l_line_w/2, subst_h];
stop = [feed1_x+flen, l_line_w/2, subst_h];
CSX = AddBox(CSX, 'Metal', 500, start, stop);

#geometry - feed
start = [feed1_x, -feed_w/2, subst_h];
stop = [feed1_x+feed_l, feed_w/2, subst_h];
CSX = AddBox(CSX, 'Metal', 500, start, stop);
start = [feed2_x, -feed_w/2, subst_h];
stop = [feed2_x+feed_l, feed_w/2, subst_h];
CSX = AddBox(CSX, 'Metal', 500, start, stop);

taper(1,1) = feed1_x+feed_l;
taper(2,1) = -feed_w/2;
taper(1,2) = feed1_x+feed_l;
taper(2,2) = feed_w/2;
taper(1,3) = feed1_x+feed_l+taper_w;
taper(2,3) = l_line_w/2;
taper(1,4) = feed1_x+feed_l+taper_w;
taper(2,4) = -l_line_w/2;
CSX = AddPolygon(CSX, 'Metal', 500, 'z', subst_h, taper);
taper(1,1) = feed2_x;
taper(2,1) = -feed_w/2;
taper(1,2) = feed2_x;
taper(2,2) = feed_w/2;
taper(1,3) = feed2_x-taper_w;
taper(2,3) = l_line_w/2;
taper(1,4) = feed2_x-taper_w;
taper(2,4) = -l_line_w/2;
CSX = AddPolygon(CSX, 'Metal', 500, 'z', subst_h, taper);

#substrate
start = [-boxsize_x/2, -boxsize_y/2, 0];
stop = [boxsize_x/2, boxsize_y/2, subst_h];
CSX = AddBox(CSX, 'FR4', 400, start, stop);

#setup mesh
xr1a = r1_x-r1_cap_w/2;
xr1b = r1_x+r1_cap_w/2;
xr1la = r1_x-r1_line_w/2;
xr1lb = r1_x+r1_line_w/2;
xr2a = r2_x-r2_cap_w/2;
xr2b = r2_x+r2_cap_w/2;
xr2la = r2_x-r2_line_w/2;
xr2lb = r2_x+r2_line_w/2;
xr3a = r3_x-r3_cap_w/2;
xr3b = r3_x+r3_cap_w/2;
xr3la = r3_x-r3_line_w/2;
xr3lb = r3_x+r3_line_w/2;
xr4a = r4_x-r4_cap_w/2;
xr4b = r4_x+r4_cap_w/2;
xr4la = r4_x-r4_line_w/2;
xr4lb = r4_x+r4_line_w/2;
xr5a = r5_x-r5_cap_w/2;
xr5b = r5_x+r5_cap_w/2;
xr5la = r5_x-r5_line_w/2;
xr5lb = r5_x+r5_line_w/2;
mesh.x = SmoothMeshLines([-flen/2, flen/2, ...
xr1a-2/3*res_f, xr1a+1/3*res_f, xr1b-1/3*res_f, xr1b+2/3*res_f, ...
xr2a-2/3*res_f, xr2a+1/3*res_f, xr2b-1/3*res_f, xr2b+2/3*res_f, ...
xr3a-2/3*res_f, xr3a+1/3*res_f, xr3b-1/3*res_f, xr3b+2/3*res_f, ...
xr4a-2/3*res_f, xr4a+1/3*res_f, xr4b-1/3*res_f, xr4b+2/3*res_f, ...
xr5a-2/3*res_f, xr5a+1/3*res_f, xr5b-1/3*res_f, xr5b+2/3*res_f, ...
xr1la-2/3*res_f, xr1la+1/3*res_f, xr1lb-1/3*res_f, xr1lb+2/3*res_f, ...
xr2la-2/3*res_f, xr2la+1/3*res_f, xr2lb-1/3*res_f, xr2lb+2/3*res_f, ...
xr3la-2/3*res_f, xr3la+1/3*res_f, xr3lb-1/3*res_f, xr3lb+2/3*res_f, ...
xr4la-2/3*res_f, xr4la+1/3*res_f, xr4lb-1/3*res_f, xr4lb+2/3*res_f, ...
xr5la-2/3*res_f, xr5la+1/3*res_f, xr5lb-1/3*res_f, xr5lb+2/3*res_f, ...
], res_f, 1.5);
mesh.x = SmoothMeshLines([-boxsize_x/2, mesh.x, boxsize_x/2], res, 1.5);

lwu = l_line_w/2;
lwl = -lwu;
r1a = lwl-r1_line_l;
r1b = r1a-r1_cap_l;
r2a = lwu+r2_line_l;
r2b = r2a+r2_cap_l;
r3a = lwl-r3_line_l;
r3b = r3a-r3_cap_l;
r4a = lwu+r4_line_l;
r4b = r4a+r4_cap_l;
r5a = lwl-r5_line_l;
r5b = r5a-r5_cap_l;
yrmin = min([r1a, r1b, r2a, r2b, r3a, r3b, r4a, r4b]);
yrmax = max([r1a, r1b, r2a, r2b, r3a, r3b, r4a, r4b]);
mesh.y = SmoothMeshLines([lwl-2/3*res_f, lwl+1/3*res_f, lwu+2/3*res_f, lwu-1/3*res_f, ...
                      r1a-1/3*res_f, r1a+2/3*res_f, ...#r1b-2/3*res_f, r1b+1/3*res_f, ...
                      r2a-2/3*res_f, r2a+1/3*res_f, r2b-1/3*res_f, r2b+2/3*res_f, ...
                      r3a-1/3*res_f, r3a+2/3*res_f, r3b-2/3*res_f, r3b+1/3*res_f, ...
                      r4a-2/3*res_f, r4a+1/3*res_f, r4b-1/3*res_f, r4b+2/3*res_f, ...
                      r5a-1/3*res_f, r5a+2/3*res_f, r5b-2/3*res_f, r5b+1/3*res_f, ...
                      yrmin-0.5e-3, yrmax+0.5e-3], res_f, 1.5);
mesh.y = SmoothMeshLines([-boxsize_y/2, mesh.y, boxsize_y/2], res, 1.5);
mesh.z = SmoothMeshLines([0, subst_h, 2*subst_h], res_f, 1.5);
mesh.z = SmoothMeshLines([mesh.z, boxsize_z], res, 1.5);
CSX = DefineRectGrid(CSX, 1, mesh);


#add ports
CSX = AddMetal( CSX, 'PEC' );
portstart = [ mesh.x(1), -feed_w/2, subst_h];
portstop  = [ -flen/2,  feed_w/2, 0];
[CSX,port{1}] = AddMSLPort( CSX, 999, 1, 'PEC', portstart, portstop, 0, [0 0 -1], 'MeasPlaneShift', boxsize_x/2-flen/2);
portstart = [ mesh.x(end), -feed_w/2, subst_h];
portstop  = [ flen/2,  feed_w/2, 0];
[CSX,port{2}] = AddMSLPort( CSX, 999, 2, 'PEC', portstart, portstop, 0, [0 0 -1], 'ExcitePort', 'excite', 'FeedShift', 10*res, 'MeasPlaneShift', boxsize_x/2-flen/2);


###############################
## PREPARE AND SIMULATE
###############################

#set files, paths, write data, show geometry, run the simulator
confirm_recursive_rmdir(0); #don't want asking when remomoving directory
sim_path = 'sim_path_filter_1_a__';
sim_file = 'sim_file.xml';
rmdir(sim_path, 's');
mkdir(sim_path);
#save structures with data about simulation to file
WriteOpenEMS([sim_path '/' sim_file], FDTD, CSX);
#run a geometry viewer
CSXGeomPlot([sim_path '/' sim_file]);
#simulate
RunOpenEMS(sim_path, sim_file);


###############################
## PROCESS RESULTS
###############################

f = linspace(fmin, fmax, 2000);
port = calcPort(port, sim_path, f, 'RefImpedance', impedance);
s22 = port{2}.uf.ref./ port{2}.uf.inc;
s12 = port{1}.uf.ref./ port{2}.uf.inc;
plot(f/1e9,20*log10(abs(s11)),'k-','LineWidth',2);
hold on;
grid on;
plot(f/1e9,20*log10(abs(s21)),'r-','LineWidth',2);
legend('S_{11}','S_{21}');
ylabel('S-Parameter (dB)','FontSize',12);
xlabel('frequency (GHz) \rightarrow','FontSize',12);
ylim([-70 0]);


















##########################
##########################
##########################
##########################

spara(1,1,:) = s11;
spara(2,1,:) = s21;
spara(1,2,:) = s12;
spara(2,2,:) = s22;
write_touchstone('s',f,spara,"filter.s2p");