close all
clear
clc

addpath('/home/michal/opt/openEMS/share/openEMS/matlab');
addpath('/home/michal/opt/openEMS/share/CSXCAD/matlab');
addpath('/home/michal/opt/openEMS/share/hyp2mat/matlab');
addpath('/home/michal/opt/openEMS/share/CTB/matlab');
physical_constants; #import physical_constants

###############################
## PREPARE SIMULATION MODEL
###############################

#variables

fmin = 0.1e9;
fmax = 10e9;
impedance = 50;
eps_r = 4.5; #FR-4 permability
tan_loss = 0.02; #FR-4 losses (as tangens)
kap = tan_loss*2*pi*((fmin*fmax)^0.5)*EPS0*eps_r; #FR-4 losses (as kappa)
res = c0 / (fmax*sqrt(eps_r)) / 60;
res_f = res / 2;
res_vf = res_f / 2;

subst_h = 1e-3;
boxsize_x = 17e-3;
boxsize_y = 11e-3;
boxsize_z = subst_h*4;

port_pos_x = 0;
line_w = 0.2e-3;
line_l = 2.41e-3;
cap_w = 4.5e-3;
cap_l = 2.6e-3;
l_ref = 1.813e-9;
c_ref = 0.916e-12;
feedW = 0.2e-3;

#initiate and setup simulation structures and parameters
FDTD = InitFDTD( 'EndCriteria', 1e-3 );
FDTD = SetGaussExcite(FDTD, fmin, fmax);
BC = {'PML_8' 'PML_8' 'PML_8' 'PML_8' 'PEC' 'PML_8'};
FDTD = SetBoundaryCond(FDTD, BC);
CSX = InitCSX();

#definitions of used materials
CSX = AddMaterial(CSX, 'FR4');
CSX = SetMaterialProperty(CSX, 'FR4', 'Epsilon', eps_r, 'Kappa', kap);
CSX = AddMetal(CSX,'Metal', 56e6, 16e-6);


#3d model:
start = [port_pos_x, -line_w/2, subst_h];
stop = [port_pos_x + line_l, line_w/2, subst_h];
CSX = AddBox(CSX, 'Metal', 500, start, stop);
start = [port_pos_x + line_l, -cap_w/2, subst_h];
stop = [port_pos_x + line_l + cap_l, cap_w/2, subst_h];
CSX = AddBox(CSX, 'Metal', 500, start, stop);
#CSX = AddMRStub( CSX, 'Metal', 500, line_w, cap_l, 90, 1, 0, [0 0 1], [line_l 0 subst_h] );


#start = [port_pos_x, -line_w/2, subst_h];
#stop = [port_pos_x - line_l, line_w/2, subst_h];
#CSX = AddBox(CSX, 'Metal', 500, start, stop);
#start = [port_pos_x - line_l, -cap_w/2, subst_h];
#stop = [port_pos_x - line_l - cap_l, cap_w/2, subst_h];
#CSX = AddBox(CSX, 'Metal', 500, start, stop);

#start = [port_pos_x, -0.3e-3, 0];
#stop = [port_pos_x-2e-3, 0.3e-3, 0.7e-3];
#CSX = AddBox(CSX, 'Metal', 500, start, stop);

#substrate
start = [-boxsize_x/2, -boxsize_y/2, 0];
stop = [boxsize_x/2, boxsize_y/2, subst_h];
CSX = AddBox(CSX, 'FR4', 400, start, stop);


#setup mesh
mesh.x = SmoothMeshLines([port_pos_x - 0.5e-3, port_pos_x, port_pos_x+line_l-2/3*res_f, port_pos_x+line_l+1/3*res_f, port_pos_x+line_l+cap_l-1/3*res_f, port_pos_x+line_l+cap_l+2/3*res_f, port_pos_x + line_l + cap_l + 0.5e-3], res_f, 1.5);
mesh.x = SmoothMeshLines([-boxsize_x/2, mesh.x, boxsize_x/2], res, 1.5);
mesh.y = SmoothMeshLines([-line_w/2-2/3*res_vf, -line_w/2+1/3*res_vf, line_w/2+2/3*res_vf, line_w/2-1/3*res_vf], res_vf, 1.5);
mesh.y = SmoothMeshLines([-cap_w/2-0.5e-3, -cap_w/2-2/3*res_f, -cap_w/2+1/3*res_f, mesh.y, cap_w/2-1/3*res_f, cap_w/2+2/3*res_f, cap_w/2+0.5e-3], res_f, 1.5);
mesh.y = SmoothMeshLines([-boxsize_y/2, mesh.y, boxsize_y/2], res, 1.5);
mesh.z = SmoothMeshLines([0, subst_h, 2*subst_h], res_f, 1.5);
mesh.z = SmoothMeshLines([mesh.z, boxsize_z], res, 1.5);
CSX = DefineRectGrid(CSX, 1, mesh);


#add ports
start = [port_pos_x, -line_w/2, 0e-3];
stop = [port_pos_x, line_w/2, subst_h];
#[CSX,port{1}] = AddLumpedPort(CSX, 999, 1, impedance, start, stop, [0 0 1], true);


CSX = AddMetal( CSX, 'PEC' );
portstart = [ mesh.x(1), -feedW/2, subst_h];
portstop  = [ 0,  feedW/2, 0];
[CSX,port{1}] = AddMSLPort( CSX, 999, 1, 'PEC', portstart, portstop, 0, [0 0 -1], 'ExcitePort', 'excite', 'FeedShift', 10*res, 'MeasPlaneShift', boxsize_x/2);

#CSX = AddLumpedElement( CSX, 'res', 2, 'R', 50);
#CSX = AddBox( CSX, 'res', 100, [0 -feedW/2 0], [0 feedW/2 subst_h] );

###############################
## PREPARE AND SIMULATE
###############################

#set files, paths, write data, show geometry, run the simulator
confirm_recursive_rmdir(0); #don't want asking when remomoving directory
sim_path = 'sim_path_lc2';
sim_file = 'sim_file.xml';
rmdir(sim_path, 's');
mkdir(sim_path);
#save structures with data about simulation to file
WriteOpenEMS([sim_path '/' sim_file], FDTD, CSX);
#run a geometry viewer
CSXGeomPlot([sim_path '/' sim_file]);
#simulate
RunOpenEMS(sim_path, sim_file);


###############################
## PROCESS RESULTS
###############################

f = linspace(fmin, fmax, 2000);
port = calcPort(port, sim_path, f, 'RefImpedance', impedance);
z_stub = abs(port{1}.uf.tot ./ port{1}.if.tot);
zp_stub = arg(port{1}.uf.tot ./ port{1}.if.tot);
z_nom = abs(i*2*pi*f*l_ref + 1 ./ (i*2*pi*f*c_ref));
zp_nom = arg(i*2*pi*f*l_ref + 1 ./ (i*2*pi*f*c_ref));
figure(1);
hold on;
loglog(f/1e9, z_stub, 'k-', 'LineWidth', 2);
loglog(f/1e9, z_nom, 'y-', 'LineWidth', 2);
grid on;
figure(2);
hold on;
semilogx(f/1e9, zp_stub, 'k-', 'LineWidth', 2);
semilogx(f/1e9, zp_nom, 'y-', 'LineWidth', 2);
grid on;
